"""@package docstring
B-ASIC GUI Module.
This python file is an example of how a GUI can be implemented
using buttons and textboxes.
"""

import sys

from PyQt5.QtWidgets import QApplication, QWidget, QMainWindow, QLabel, QAction,\
QStatusBar, QMenuBar, QLineEdit, QPushButton, QSlider, QScrollArea, QVBoxLayout,\
QHBoxLayout, QDockWidget, QToolBar, QMenu
from PyQt5.QtCore import Qt, QSize, pyqtSlot
from PyQt5.QtGui import QIcon, QFont, QPainter, QPen, QColor

from b_asic.core_operations import Addition


class DragButton(QPushButton):
    def __init__(self, name, window, parent = None):
        self.name = name
        self.__menu = None
        self.__window = window
        self.counter = 0
        self.clicked = 0
        self.pressed = False
        print("Constructor" + self.name)
        super(DragButton, self).__init__(self.__window)

        self.__window.setContextMenuPolicy(Qt.CustomContextMenu)
        self.__window.customContextMenuRequested.connect(self.create_menu) 
        

    @pyqtSlot(QAction)
    def actionClicked(self, action):
        print("Triggern "+ self.name, self.__menu.name)
        #self.__window.check_for_remove_op(self.name)

    #def show_context_menu(self, point):
        # show context menu
        

    def create_menu(self, point):
        self.counter += 1
        # create context menu
        popMenu = MyMenu('Menu' + str(self.counter))
        popMenu.addAction(QAction('Add a signal', self))
        popMenu.addAction(QAction('Remove a signal', self))
        #action = QAction('Remove operation', self)
        popMenu.addAction('Remove operation', lambda:self.removeAction(self))
        popMenu.addSeparator()
        popMenu.addAction(QAction('Remove all signals', self))
        self.__window.menuList.append(popMenu)
        #self.__window.actionList.append(action)
        self.__menu = popMenu
        self.pressed = False
        self.__menu.exec_(self.__window.sender().mapToGlobal(point))
        self.__menu.triggered.connect(self.actionClicked)


    def removeAction(self, op):
        print(op.__menu.name, op.name)
        op.remove()

    """This class is made to create a draggable button"""

    def mousePressEvent(self, event):
        self._mouse_press_pos = None
        self._mouse_move_pos = None
        self.clicked += 1
        if self.clicked == 1:
            self.pressed = True
            self.setStyleSheet("background-color: grey; border-style: solid;\
            border-color: black; border-width: 2px; border-radius: 10px")
        elif self.clicked == 2:
            self.clicked = 0
            self.presseed = False
            self.setStyleSheet("background-color: white; border-style: solid;\
            border-color: black; border-width: 2px; border-radius: 10px")
        
        if event.button() == Qt.LeftButton:
            self._mouse_press_pos = event.globalPos()
            self._mouse_move_pos = event.globalPos()

        super(DragButton, self).mousePressEvent(event)

    def mouseMoveEvent(self, event):
        if event.buttons() == Qt.LeftButton:
            cur_pos = self.mapToGlobal(self.pos())
            global_pos = event.globalPos()
            diff = global_pos - self._mouse_move_pos
            new_pos = self.mapFromGlobal(cur_pos + diff)
            self.move(new_pos)
            self.pressed = False

            self._mouse_move_pos = global_pos

        super(DragButton, self).mouseMoveEvent(event)

    def mouseReleaseEvent(self, event):
        if self._mouse_press_pos is not None:
            moved = event.globalPos() - self._mouse_press_pos
            if moved.manhattanLength() > 3:
                event.ignore()
                return
      
        super(DragButton, self).mouseReleaseEvent(event)

    def remove(self):
        self.deleteLater()

class SubWindow(QWidget):
    """Creates a sub window """
    def create_window(self, window_width, window_height):
        """Creates a window
        """
        parent = None
        super(SubWindow, self).__init__(parent)
        self.setWindowFlags(Qt.WindowStaysOnTopHint)
        self.resize(window_width, window_height)


class MyMenu(QMenu):
    def __init__(self, name, parent = None):
        self.name = name
        super(MyMenu, self).__init__()
    

class MainWindow(QMainWindow):
    """Main window for the program"""
    # pylint: disable=too-many-instance-attributes
    # Eight is reasonable in this case.
    def __init__(self, *args, **kwargs):
        super(MainWindow, self).__init__(*args, **kwargs)
        self.init_ui()
        self.counter = 0
        self.operations = []
        self.menuList = []
        self.actionList = []

    def init_ui(self):
        self.setWindowTitle(" ")
        self.setWindowIcon(QIcon('small_logo.png'))
        self.create_operation_menu()
        self.create_menu_bar()
        self.setStatusBar(QStatusBar(self))

    def create_operation_menu(self):
        self.operation_box = QDockWidget("Operation Box", self)
        self.operation_box.setAllowedAreas(Qt.LeftDockWidgetArea)
        self.test = QToolBar(self)
        self.operation_list = QMenuBar(self)
        self.test.addWidget(self.operation_list)
        self.test.setOrientation(Qt.Vertical)
        self.operation_list.setStyleSheet("background-color:rgb(222,222,222); vertical")
        basic_operations = self.operation_list.addMenu('Basic operations')
        special_operations = self.operation_list.addMenu('Special operations')

        self.addition_menu_item = QAction('&Addition', self)
        self.addition_menu_item.setStatusTip("Add addition operation to workspace")
        self.addition_menu_item.triggered.connect(self.create_addition_operation)
        basic_operations.addAction(self.addition_menu_item)

        self.subtraction_menu_item = QAction('&Subtraction', self)
        self.subtraction_menu_item.setStatusTip("Add subtraction operation to workspace")
        self.subtraction_menu_item.triggered.connect(self.create_subtraction_operation)
        basic_operations.addAction(self.subtraction_menu_item)

        self.multiplication_menu_item = QAction('&Multiplication', self)
        self.multiplication_menu_item.setStatusTip("Add multiplication operation to workspace")
        self.multiplication_menu_item.triggered.connect(self.create_multiplication_operation)
        basic_operations.addAction(self.multiplication_menu_item)

        self.division_menu_item = QAction('&Division', self)
        self.division_menu_item.setStatusTip("Add division operation to workspace")
        #self.division_menu_item.triggered.connect(self.create_division_operation)
        basic_operations.addAction(self.division_menu_item)

        self.constant_menu_item = QAction('&Constant', self)
        self.constant_menu_item.setStatusTip("Add constant operation to workspace")
        #self.constant_menu_item.triggered.connect(self.create_constant_operation)
        basic_operations.addAction(self.constant_menu_item)

        self.square_root_menu_item = QAction('&Square root', self)
        self.square_root_menu_item.setStatusTip("Add square root operation to workspace")
        #self.square_root_menu_item.triggered.connect(self.create_square_root_operation)
        basic_operations.addAction(self.square_root_menu_item)

        self.complex_conjugate_menu_item = QAction('&Complex conjugate', self)
        self.complex_conjugate_menu_item.setStatusTip("Add complex conjugate operation to workspace")
        #self.complex_conjugate_menu_item.triggered.connect(self.create_complex_conjugate_operation)
        basic_operations.addAction(self.complex_conjugate_menu_item)

        self.max_menu_item = QAction('&Max', self)
        self.max_menu_item.setStatusTip("Add max operation to workspace")
        #self.max_menu_item.triggered.connect(self.create_max_operation)
        basic_operations.addAction(self.max_menu_item)

        self.min_menu_item = QAction('&Min', self)
        self.min_menu_item.setStatusTip("Add min operation to workspace")
        #self.min_menu_item.triggered.connect(self.create_min_operation)
        basic_operations.addAction(self.min_menu_item)

        self.absolute_menu_item = QAction('&Absolute', self)
        self.absolute_menu_item.setStatusTip("Add absolute operation to workspace")
        #self.absolute_menu_item.triggered.connect(self.create_absolute_operation)
        basic_operations.addAction(self.absolute_menu_item)

        self.constant_addition_menu_item = QAction('&Constant addition', self)
        self.constant_addition_menu_item.setStatusTip("Add constant addition operation to workspace")
        #self.constant_addition_menu_item.triggered.connect(self.create_constant_addition_operation)
        basic_operations.addAction(self.constant_addition_menu_item)

        self.constant_subtraction_menu_item = QAction('&Constant subtraction', self)
        self.constant_subtraction_menu_item.setStatusTip("Add constant subtraction operation to workspace")
        #self.constant_subtraction_menu_item.triggered.connect(self.create_constant_subtraction_operation)
        basic_operations.addAction(self.constant_subtraction_menu_item)

        self.constant_multiplication_menu_item = QAction('&Constant multiplication', self)
        self.constant_multiplication_menu_item.setStatusTip("Add constant multiplication operation to workspace")
        #self.constant_multiplication_menu_item.triggered.connect(self.create_constant_multiplication_operation)
        basic_operations.addAction(self.constant_multiplication_menu_item)

        self.constant_division_menu_item = QAction('&Constant division', self)
        self.constant_division_menu_item.setStatusTip("Add constant division operation to workspace")
        #self.constant_division_menu_item.triggered.connect(self.create_constant_division_operation)
        basic_operations.addAction(self.constant_division_menu_item)

        self.butterfly_menu_item = QAction('&Butterfly', self)
        self.butterfly_menu_item.setStatusTip("Add butterfly operation to workspace")
        #self.butterfly_menu_item.triggered.connect(self.create_butterfly_operation)
        basic_operations.addAction(self.butterfly_menu_item)

        self.operation_box.setWidget(self.operation_list)
        self.operation_box.setMaximumSize(240, 400)
        self.operation_box.setFeatures(QDockWidget.NoDockWidgetFeatures)
        self.operation_box.setFixedSize(300, 500)
        self.operation_box.setStyleSheet("background-color: white; border-style: solid;\
            border-color: black; border-width: 2px")
        self.addDockWidget(Qt.LeftDockWidgetArea, self.operation_box)

    def create_addition_operation(self):
        self.counter += 1
        
        # Create drag button
        addition_operation = DragButton("OP" + str(self.counter), self)
        addition_operation.move(250, 100)
        addition_operation.setFixedSize(50, 50)
        addition_operation.setStyleSheet("background-color: white; border-style: solid;\
            border-color: black; border-width: 2px; border-radius: 10px")
        addition_operation.clicked.connect(self.create_sub_window)
        #self.addition_operation.setIcon(QIcon("GUI'\'operation_icons'\'plus.png"))
        addition_operation.setText("OP" + str(self.counter))
        addition_operation.setIconSize(QSize(50, 50))
        addition_operation.show()
        self.operations.append(addition_operation)
        
        # set context menu policies
        #self.addition_operation.setContextMenuPolicy(Qt.CustomContextMenu)
        #self.addition_operation.customContextMenuRequested.connect(self.show_context_menu)

        #self.action.triggered.connect(lambda checked: self.remove(self.addition_operation.name))

    def check_for_remove_op(self, name):
        self.remove(name) 
        
    
    def remove(self, name):
        for op in self.operations:
            print(name, op.name)
            if op.name == name:
                self.operations.remove(op)
                op.remove()

    def create_subtraction_operation(self):
        self.subtraction_operation = DragButton("sub" + str(self.counter), self)
        self.subtraction_operation.move(250, 100)
        self.subtraction_operation.setFixedSize(50, 50)
        self.subtraction_operation.setStyleSheet("background-color: white; border-style: solid;\
            border-color: black; border-width: 2px; border-radius: 10px")
        self.subtraction_operation.setIcon(QIcon("GUI'\'operation_icons'\'minus.png"))
        self.subtraction_operation.setIconSize(QSize(50, 50))
        self.subtraction_operation.clicked.connect(self.create_sub_window)
        self.subtraction_operation.show()

         # set context menu policies
        self.subtraction_operation.setContextMenuPolicy(Qt.CustomContextMenu)
        self.subtraction_operation.customContextMenuRequested.connect(self.show_context_menu)

        # create context menu
        self.button_context_menu = QMenu(self)
        self.button_context_menu.addAction(QAction('Add a signal', self))
        self.button_context_menu.addAction(QAction('Remove a signal', self))
        self.button_context_menu.addSeparator()
        self.button_context_menu.addAction(QAction('Remove all signals', self))

    def create_multiplication_operation(self):
        self.multiplication_operation = DragButton(self)
        self.multiplication_operation.move(250, 100)
        self.multiplication_operation.setFixedSize(50, 50)
        self.multiplication_operation.setStyleSheet("background-color: white; border-style: solid;\
            border-color: black; border-width: 2px; border-radius: 10px")
        self.multiplication_operation.clicked.connect(self.create_sub_window)
        self.multiplication_operation.setIcon(QIcon(r"GUI\operation_icons\plus.png"))
        self.multiplication_operation.setIconSize(QSize(50, 50))
        self.multiplication_operation.show()

         # set context menu policies
        self.multiplication_operation.setContextMenuPolicy(Qt.CustomContextMenu)
        self.multiplication_operation.customContextMenuRequested.connect(self.show_context_menu)

        # create context menu
        self.button_context_menu = QMenu(self)
        self.button_context_menu.addAction(QAction('Add a signal', self))
        self.button_context_menu.addAction(QAction('Remove a signal', self))
        self.button_context_menu.addSeparator()
        self.button_context_menu.addAction(QAction('Remove all signals', self))
    
    
    def create_menu_bar(self):
        # Menu buttons
        load_button = QAction("Load", self)
        save_button = QAction("Save", self)

        exit_button = QAction("Exit", self)
        exit_button.setShortcut("Ctrl+Q")
        exit_button.triggered.connect(self.exit_app)

        edit_button = QAction("Edit", self)
        edit_button.setStatusTip("Open edit menu")
        edit_button.triggered.connect(self.on_edit_button_click)

        view_button = QAction("View", self)
        view_button.setStatusTip("Open view menu")
        view_button.triggered.connect(self.on_view_button_click)

        menu_bar = QMenuBar()
        menu_bar.setStyleSheet("background-color:rgb(222, 222, 222)")
        self.setMenuBar(menu_bar)

        file_menu = menu_bar.addMenu("&File")
        file_menu.addAction(save_button)
        file_menu.addSeparator()
        file_menu.addAction(exit_button)

        edit_menu = menu_bar.addMenu("&Edit")
        edit_menu.addAction(edit_button)

        edit_menu.addSeparator()

        view_menu = menu_bar.addMenu("&View")
        view_menu.addAction(view_button)


    def create_sub_window(self):
        """ Example of how to create a sub window
        """
        self.sub_window = SubWindow()
        self.sub_window.create_window(400, 300)
        self.sub_window.setWindowTitle("Properties")

        self.sub_window.properties_label = QLabel(self.sub_window)
        self.sub_window.properties_label.setText('Properties')
        self.sub_window.properties_label.setFixedWidth(400)
        self.sub_window.properties_label.setFont(QFont('SansSerif', 14, QFont.Bold))
        self.sub_window.properties_label.setAlignment(Qt.AlignCenter)

        self.sub_window.name_label = QLabel(self.sub_window)
        self.sub_window.name_label.setText('Name:')
        self.sub_window.name_label.move(20, 40)

        self.sub_window.name_line = QLineEdit(self.sub_window)
        self.sub_window.name_line.setPlaceholderText("Write a name here")
        self.sub_window.name_line.move(70, 40)
        self.sub_window.name_line.resize(100, 20)

        self.sub_window.id_label = QLabel(self.sub_window)
        self.sub_window.id_label.setText('Id:')
        self.sub_window.id_label.move(20, 70)

        self.sub_window.id_line = QLineEdit(self.sub_window)
        self.sub_window.id_line.setPlaceholderText("Write an id here")
        self.sub_window.id_line.move(70, 70)
        self.sub_window.id_line.resize(100, 20)

        self.sub_window.show()

    def keyPressEvent(self, event):
        for op in self.operations:
            if event.key() == Qt.Key_Delete and op.pressed:
                self.operations.remove(op)
                op.remove()
    
    def on_file_button_click(self):
        print("File")

    def on_edit_button_click(self):
        print("Edit")

    def on_view_button_click(self):
        print("View")

    def exit_app(self, checked):
        QApplication.quit()

    def clicked(self):
        print("Drag button clicked")
            

if __name__ == "__main__":
    app = QApplication(sys.argv)
    window = MainWindow()
    window.resize(960, 720)
    window.show()
    app.exec_()
